import React from 'react';
import {
  Text,
  View,
  TouchableWithoutFeedback,
} from 'react-native';
import htmlparser from 'htmlparser2-without-node-native';
import entities from 'entities';

import AutoSizedImage from './AutoSizedImage';

/**
 * 需要数字化的 属性
 * @type {[string,string,string]}
 */
const NUMBER_PROPS = ['fontSize', 'marginTop', 'lineHeight', 'height', 'width', 'paddingRight', 'marginRight', 'marginLight', 'marginTop', 'marginBottom'];

const LINE_BREAK = '\n';
const PARAGRAPH_BREAK = '\n\n';
const BULLET = '\u2022 ';

const Img = props => {
  const width = Number(props.attribs['width']) || Number(props.attribs['data-width']) || 0;
  const height = Number(props.attribs['height']) || Number(props.attribs['data-height']) || 0;

  const imgStyle = {
    width,
    height,
  };
  const source = {
    uri: props.attribs.src,
    width,
    height,
  };
  return (
    <AutoSizedImage source={source} style={imgStyle} />
  );
};

export default function htmlToElement(rawHtml, opts, done) {
  function domToElement(dom, parent) {
    if (!dom) return null;

    return dom.map((node, index, list) => {
      if (opts.customRenderer) {
        const rendered = opts.customRenderer(node, index, list, parent, domToElement);
        if (rendered || rendered === null) return rendered;
      }

      if (node.type == 'text') {
        let txtStyle = {};
        if (parent){
          let parentattribs =  parent.attribs;
          if(parentattribs && typeof parentattribs === 'object'){
              for(let key in parentattribs){
                let humpKey = key.hyphenation2hump();
                txtStyle[humpKey] = NUMBER_PROPS.contains(humpKey) ? Number(parentattribs[key]) : parentattribs[key];
              }
          }
        }

        return (
          <Text key={index} style={[parent ? opts.styles[parent.name] : null, txtStyle ? txtStyle : null]}>
            {entities.decodeHTML(node.data)}
          </Text>
        );
      }

      if (node.type == 'tag') {
        if (node.name == 'img') {
          return (
            <Img key={index} attribs={node.attribs} />
          );
        }

        if(node.name == 'div'){
            let viewStyle = {};
            let attribs =  node.attribs;
            if(attribs && typeof attribs === 'object'){
                for(let key in attribs){
                    let humpKey = key.hyphenation2hump();
                    viewStyle[humpKey] = NUMBER_PROPS.contains(humpKey) ? Number(attribs[key]) : attribs[key];
                }
            }
            return (
                <View key={index} style={[viewStyle]}>
                    {domToElement(node.children, node)}
                </View>
            );
        }

        let linkPressHandler = null;
        if (node.name == 'a' && node.attribs && node.attribs.href) {
          linkPressHandler = () => opts.linkHandler(entities.decodeHTML(node.attribs.href));
        }

        let linebreakBefore = null;
        let linebreakAfter = null;
        if (opts.addLineBreaks) {
          switch (node.name) {
          case 'pre':
            linebreakBefore = LINE_BREAK;
            break;
          case 'p':
            if (index < list.length - 1) {
              linebreakAfter = PARAGRAPH_BREAK;
            }
            break;
          case 'br':
          case 'h1':
          case 'h2':
          case 'h3':
          case 'h4':
          case 'h5':
            linebreakAfter = LINE_BREAK;
            break;
          }
        }

        let listItemPrefix = null;
        if (node.name == 'li') {
          if (parent.name == 'ol') {
            listItemPrefix = `${index + 1}. `;
          } else if (parent.name == 'ul') {
            listItemPrefix = BULLET;
          }
        }

        let contentInnerView = (
            <View key={index}>
                {linebreakBefore}
                {listItemPrefix}
                {domToElement(node.children, node)}
                {linebreakAfter}
            </View>
        );

        return (
            linkPressHandler ?
              (
                  <TouchableWithoutFeedback key={index} onPress={linkPressHandler}>
                      {contentInnerView}
                  </TouchableWithoutFeedback>
              )
              :
              contentInnerView
        );
      }
    });
  }

  const handler = new htmlparser.DomHandler(function(err, dom) {
    if (err) done(err);
    done(null, domToElement(dom));
  });
  const parser = new htmlparser.Parser(handler);
  parser.write(rawHtml);
  parser.done();
}
